# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class Firewall(pulumi.CustomResource):
    created_at: pulumi.Output[str]
    """
    A time value given in ISO8601 combined date and time format
    that represents when the Firewall was created.
    """
    droplet_ids: pulumi.Output[list]
    """
    The list of the IDs of the Droplets assigned
    to the Firewall.
    """
    inbound_rules: pulumi.Output[list]
    """
    The inbound access rule block for the Firewall.
    The `inbound_rule` block is documented below.

      * `portRange` (`str`) - The ports on which traffic will be allowed
        specified as a string containing a single port, a range (e.g. "8000-9000"),
        or "1-65535" to open all ports for a protocol. Required for when protocol is
        `tcp` or `udp`.
      * `protocol` (`str`) - The type of traffic to be allowed.
        This may be one of "tcp", "udp", or "icmp".
      * `sourceAddresses` (`list`) - An array of strings containing the IPv4
        addresses, IPv6 addresses, IPv4 CIDRs, and/or IPv6 CIDRs from which the
        inbound traffic will be accepted.
      * `sourceDropletIds` (`list`) - An array containing the IDs of
        the Droplets from which the inbound traffic will be accepted.
      * `sourceLoadBalancerUids` (`list`) - An array containing the IDs
        of the Load Balancers from which the inbound traffic will be accepted.
      * `sourceTags` (`list`) - An array containing the names of Tags
        corresponding to groups of Droplets from which the inbound traffic
        will be accepted.
    """
    name: pulumi.Output[str]
    """
    The Firewall name
    """
    outbound_rules: pulumi.Output[list]
    """
    The outbound access rule block for the Firewall.
    The `outbound_rule` block is documented below.

      * `destinationAddresses` (`list`) - An array of strings containing the IPv4
        addresses, IPv6 addresses, IPv4 CIDRs, and/or IPv6 CIDRs to which the
        outbound traffic will be allowed.
      * `destinationDropletIds` (`list`) - An array containing the IDs of
        the Droplets to which the outbound traffic will be allowed.
      * `destinationLoadBalancerUids` (`list`) - An array containing the IDs
        of the Load Balancers to which the outbound traffic will be allowed.
      * `destinationTags` (`list`) - An array containing the names of Tags
        corresponding to groups of Droplets to which the outbound traffic will
        be allowed.
        traffic.
      * `portRange` (`str`) - The ports on which traffic will be allowed
        specified as a string containing a single port, a range (e.g. "8000-9000"),
        or "1-65535" to open all ports for a protocol. Required for when protocol is
        `tcp` or `udp`.
      * `protocol` (`str`) - The type of traffic to be allowed.
        This may be one of "tcp", "udp", or "icmp".
    """
    pending_changes: pulumi.Output[list]
    """
    An list of object containing the fields, "droplet_id",
    "removing", and "status".  It is provided to detail exactly which Droplets
    are having their security policies updated.  When empty, all changes
    have been successfully applied.

      * `droplet_id` (`float`)
      * `removing` (`bool`)
      * `status` (`str`) - A status string indicating the current state of the Firewall.
        This can be "waiting", "succeeded", or "failed".
    """
    status: pulumi.Output[str]
    """
    A status string indicating the current state of the Firewall.
    This can be "waiting", "succeeded", or "failed".
    """
    tags: pulumi.Output[list]
    """
    The names of the Tags assigned to the Firewall.
    """
    def __init__(__self__, resource_name, opts=None, droplet_ids=None, inbound_rules=None, name=None, outbound_rules=None, tags=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a DigitalOcean Cloud Firewall resource. This can be used to create,
        modify, and delete Firewalls.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/r/firewall.html.markdown.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] droplet_ids: The list of the IDs of the Droplets assigned
               to the Firewall.
        :param pulumi.Input[list] inbound_rules: The inbound access rule block for the Firewall.
               The `inbound_rule` block is documented below.
        :param pulumi.Input[str] name: The Firewall name
        :param pulumi.Input[list] outbound_rules: The outbound access rule block for the Firewall.
               The `outbound_rule` block is documented below.
        :param pulumi.Input[list] tags: The names of the Tags assigned to the Firewall.

        The **inbound_rules** object supports the following:

          * `portRange` (`pulumi.Input[str]`) - The ports on which traffic will be allowed
            specified as a string containing a single port, a range (e.g. "8000-9000"),
            or "1-65535" to open all ports for a protocol. Required for when protocol is
            `tcp` or `udp`.
          * `protocol` (`pulumi.Input[str]`) - The type of traffic to be allowed.
            This may be one of "tcp", "udp", or "icmp".
          * `sourceAddresses` (`pulumi.Input[list]`) - An array of strings containing the IPv4
            addresses, IPv6 addresses, IPv4 CIDRs, and/or IPv6 CIDRs from which the
            inbound traffic will be accepted.
          * `sourceDropletIds` (`pulumi.Input[list]`) - An array containing the IDs of
            the Droplets from which the inbound traffic will be accepted.
          * `sourceLoadBalancerUids` (`pulumi.Input[list]`) - An array containing the IDs
            of the Load Balancers from which the inbound traffic will be accepted.
          * `sourceTags` (`pulumi.Input[list]`) - An array containing the names of Tags
            corresponding to groups of Droplets from which the inbound traffic
            will be accepted.

        The **outbound_rules** object supports the following:

          * `destinationAddresses` (`pulumi.Input[list]`) - An array of strings containing the IPv4
            addresses, IPv6 addresses, IPv4 CIDRs, and/or IPv6 CIDRs to which the
            outbound traffic will be allowed.
          * `destinationDropletIds` (`pulumi.Input[list]`) - An array containing the IDs of
            the Droplets to which the outbound traffic will be allowed.
          * `destinationLoadBalancerUids` (`pulumi.Input[list]`) - An array containing the IDs
            of the Load Balancers to which the outbound traffic will be allowed.
          * `destinationTags` (`pulumi.Input[list]`) - An array containing the names of Tags
            corresponding to groups of Droplets to which the outbound traffic will
            be allowed.
            traffic.
          * `portRange` (`pulumi.Input[str]`) - The ports on which traffic will be allowed
            specified as a string containing a single port, a range (e.g. "8000-9000"),
            or "1-65535" to open all ports for a protocol. Required for when protocol is
            `tcp` or `udp`.
          * `protocol` (`pulumi.Input[str]`) - The type of traffic to be allowed.
            This may be one of "tcp", "udp", or "icmp".
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['droplet_ids'] = droplet_ids
            __props__['inbound_rules'] = inbound_rules
            __props__['name'] = name
            __props__['outbound_rules'] = outbound_rules
            __props__['tags'] = tags
            __props__['created_at'] = None
            __props__['pending_changes'] = None
            __props__['status'] = None
        super(Firewall, __self__).__init__(
            'digitalocean:index/firewall:Firewall',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, created_at=None, droplet_ids=None, inbound_rules=None, name=None, outbound_rules=None, pending_changes=None, status=None, tags=None):
        """
        Get an existing Firewall resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] created_at: A time value given in ISO8601 combined date and time format
               that represents when the Firewall was created.
        :param pulumi.Input[list] droplet_ids: The list of the IDs of the Droplets assigned
               to the Firewall.
        :param pulumi.Input[list] inbound_rules: The inbound access rule block for the Firewall.
               The `inbound_rule` block is documented below.
        :param pulumi.Input[str] name: The Firewall name
        :param pulumi.Input[list] outbound_rules: The outbound access rule block for the Firewall.
               The `outbound_rule` block is documented below.
        :param pulumi.Input[list] pending_changes: An list of object containing the fields, "droplet_id",
               "removing", and "status".  It is provided to detail exactly which Droplets
               are having their security policies updated.  When empty, all changes
               have been successfully applied.
        :param pulumi.Input[str] status: A status string indicating the current state of the Firewall.
               This can be "waiting", "succeeded", or "failed".
        :param pulumi.Input[list] tags: The names of the Tags assigned to the Firewall.

        The **inbound_rules** object supports the following:

          * `portRange` (`pulumi.Input[str]`) - The ports on which traffic will be allowed
            specified as a string containing a single port, a range (e.g. "8000-9000"),
            or "1-65535" to open all ports for a protocol. Required for when protocol is
            `tcp` or `udp`.
          * `protocol` (`pulumi.Input[str]`) - The type of traffic to be allowed.
            This may be one of "tcp", "udp", or "icmp".
          * `sourceAddresses` (`pulumi.Input[list]`) - An array of strings containing the IPv4
            addresses, IPv6 addresses, IPv4 CIDRs, and/or IPv6 CIDRs from which the
            inbound traffic will be accepted.
          * `sourceDropletIds` (`pulumi.Input[list]`) - An array containing the IDs of
            the Droplets from which the inbound traffic will be accepted.
          * `sourceLoadBalancerUids` (`pulumi.Input[list]`) - An array containing the IDs
            of the Load Balancers from which the inbound traffic will be accepted.
          * `sourceTags` (`pulumi.Input[list]`) - An array containing the names of Tags
            corresponding to groups of Droplets from which the inbound traffic
            will be accepted.

        The **outbound_rules** object supports the following:

          * `destinationAddresses` (`pulumi.Input[list]`) - An array of strings containing the IPv4
            addresses, IPv6 addresses, IPv4 CIDRs, and/or IPv6 CIDRs to which the
            outbound traffic will be allowed.
          * `destinationDropletIds` (`pulumi.Input[list]`) - An array containing the IDs of
            the Droplets to which the outbound traffic will be allowed.
          * `destinationLoadBalancerUids` (`pulumi.Input[list]`) - An array containing the IDs
            of the Load Balancers to which the outbound traffic will be allowed.
          * `destinationTags` (`pulumi.Input[list]`) - An array containing the names of Tags
            corresponding to groups of Droplets to which the outbound traffic will
            be allowed.
            traffic.
          * `portRange` (`pulumi.Input[str]`) - The ports on which traffic will be allowed
            specified as a string containing a single port, a range (e.g. "8000-9000"),
            or "1-65535" to open all ports for a protocol. Required for when protocol is
            `tcp` or `udp`.
          * `protocol` (`pulumi.Input[str]`) - The type of traffic to be allowed.
            This may be one of "tcp", "udp", or "icmp".

        The **pending_changes** object supports the following:

          * `droplet_id` (`pulumi.Input[float]`)
          * `removing` (`pulumi.Input[bool]`)
          * `status` (`pulumi.Input[str]`) - A status string indicating the current state of the Firewall.
            This can be "waiting", "succeeded", or "failed".
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["created_at"] = created_at
        __props__["droplet_ids"] = droplet_ids
        __props__["inbound_rules"] = inbound_rules
        __props__["name"] = name
        __props__["outbound_rules"] = outbound_rules
        __props__["pending_changes"] = pending_changes
        __props__["status"] = status
        __props__["tags"] = tags
        return Firewall(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class KubernetesCluster(pulumi.CustomResource):
    cluster_subnet: pulumi.Output[str]
    """
    The range of IP addresses in the overlay network of the Kubernetes cluster.
    """
    created_at: pulumi.Output[str]
    """
    The date and time when the Kubernetes cluster was created.
    """
    endpoint: pulumi.Output[str]
    """
    The base URL of the API server on the Kubernetes master node.
    """
    ipv4_address: pulumi.Output[str]
    """
    The public IPv4 address of the Kubernetes master node.
    """
    kube_configs: pulumi.Output[list]
    name: pulumi.Output[str]
    """
    A name for the Kubernetes cluster.
    """
    node_pool: pulumi.Output[dict]
    """
    A block representing the cluster's default node pool. Additional node pools may be added to the cluster using the `.KubernetesNodePool` resource. The following arguments may be specified:
    - `name` - (Required) A name for the node pool.
    - `size` - (Required) The slug identifier for the type of Droplet to be used as workers in the node pool.
    - `node_count` - (Optional) The number of Droplet instances in the node pool. If auto-scaling is enabled, this should only be set if the desired result is to explicitly reset the number of nodes to this value. If auto-scaling is enabled, and the node count is outside of the given min/max range, it will use the min nodes value.
    - `auto_scale` - (Optional) Enable auto-scaling of the number of nodes in the node pool within the given min/max range.
    - `min_nodes` - (Optional) If auto-scaling is enabled, this represents the minimum number of nodes that the node pool can be scaled down to.
    - `max_nodes` - (Optional) If auto-scaling is enabled, this represents the maximum number of nodes that the node pool can be scaled up to.
    - `tags` - (Optional) A list of tag names to be applied to the Kubernetes cluster.
    - `labels` - (Optional) A map of key/value pairs to apply to nodes in the pool. The labels are exposed in the Kubernetes API as labels in the metadata of the corresponding [Node resources](https://kubernetes.io/docs/concepts/architecture/nodes/).

      * `actual_node_count` (`float`)
      * `auto_scale` (`bool`)
      * `id` (`str`) - A unique ID that can be used to identify and reference a Kubernetes cluster.
      * `labels` (`dict`)
      * `max_nodes` (`float`)
      * `min_nodes` (`float`)
      * `name` (`str`) - A name for the Kubernetes cluster.
      * `node_count` (`float`)
      * `nodes` (`list`)
        * `created_at` (`str`) - The date and time when the Kubernetes cluster was created.
        * `droplet_id` (`str`)
        * `id` (`str`) - A unique ID that can be used to identify and reference a Kubernetes cluster.
        * `name` (`str`) - A name for the Kubernetes cluster.
        * `status` (`str`) - A string indicating the current status of the cluster. Potential values include running, provisioning, and errored.
        * `updated_at` (`str`) - The date and time when the Kubernetes cluster was last updated.
          * `kube_config.0` - A representation of the Kubernetes cluster's kubeconfig with the following attributes:
          - `raw_config` - The full contents of the Kubernetes cluster's kubeconfig file.
          - `host` - The URL of the API server on the Kubernetes master node.
          - `cluster_ca_certificate` - The base64 encoded public certificate for the cluster's certificate authority.
          - `token` - The DigitalOcean API access token used by clients to access the cluster.
          - `client_key` - The base64 encoded private key used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
          - `client_certificate` - The base64 encoded public certificate used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
          - `expires_at` - The date and time when the credentials will expire and need to be regenerated.

      * `size` (`str`)
      * `tags` (`list`) - A list of tag names to be applied to the Kubernetes cluster.
    """
    region: pulumi.Output[str]
    """
    The slug identifier for the region where the Kubernetes cluster will be created.
    """
    service_subnet: pulumi.Output[str]
    """
    The range of assignable IP addresses for services running in the Kubernetes cluster.
    """
    status: pulumi.Output[str]
    """
    A string indicating the current status of the cluster. Potential values include running, provisioning, and errored.
    """
    tags: pulumi.Output[list]
    """
    A list of tag names to be applied to the Kubernetes cluster.
    """
    updated_at: pulumi.Output[str]
    """
    The date and time when the Kubernetes cluster was last updated.
    * `kube_config.0` - A representation of the Kubernetes cluster's kubeconfig with the following attributes:
    - `raw_config` - The full contents of the Kubernetes cluster's kubeconfig file.
    - `host` - The URL of the API server on the Kubernetes master node.
    - `cluster_ca_certificate` - The base64 encoded public certificate for the cluster's certificate authority.
    - `token` - The DigitalOcean API access token used by clients to access the cluster.
    - `client_key` - The base64 encoded private key used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
    - `client_certificate` - The base64 encoded public certificate used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
    - `expires_at` - The date and time when the credentials will expire and need to be regenerated.
    """
    version: pulumi.Output[str]
    """
    The slug identifier for the version of Kubernetes used for the cluster. Use [doctl](https://github.com/digitalocean/doctl) to find the available versions `doctl kubernetes options versions`. (**Note:** A cluster may only be upgraded to newer versions in-place. If the version is decreased, a new resource will be created.)
    """
    def __init__(__self__, resource_name, opts=None, name=None, node_pool=None, region=None, tags=None, version=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a DigitalOcean Kubernetes cluster resource. This can be used to create, delete, and modify clusters. For more information see the [official documentation](https://www.digitalocean.com/docs/kubernetes/).

        > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/r/kubernetes_cluster.html.markdown.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] name: A name for the Kubernetes cluster.
        :param pulumi.Input[dict] node_pool: A block representing the cluster's default node pool. Additional node pools may be added to the cluster using the `.KubernetesNodePool` resource. The following arguments may be specified:
               - `name` - (Required) A name for the node pool.
               - `size` - (Required) The slug identifier for the type of Droplet to be used as workers in the node pool.
               - `node_count` - (Optional) The number of Droplet instances in the node pool. If auto-scaling is enabled, this should only be set if the desired result is to explicitly reset the number of nodes to this value. If auto-scaling is enabled, and the node count is outside of the given min/max range, it will use the min nodes value.
               - `auto_scale` - (Optional) Enable auto-scaling of the number of nodes in the node pool within the given min/max range.
               - `min_nodes` - (Optional) If auto-scaling is enabled, this represents the minimum number of nodes that the node pool can be scaled down to.
               - `max_nodes` - (Optional) If auto-scaling is enabled, this represents the maximum number of nodes that the node pool can be scaled up to.
               - `tags` - (Optional) A list of tag names to be applied to the Kubernetes cluster.
               - `labels` - (Optional) A map of key/value pairs to apply to nodes in the pool. The labels are exposed in the Kubernetes API as labels in the metadata of the corresponding [Node resources](https://kubernetes.io/docs/concepts/architecture/nodes/).
        :param pulumi.Input[str] region: The slug identifier for the region where the Kubernetes cluster will be created.
        :param pulumi.Input[list] tags: A list of tag names to be applied to the Kubernetes cluster.
        :param pulumi.Input[str] version: The slug identifier for the version of Kubernetes used for the cluster. Use [doctl](https://github.com/digitalocean/doctl) to find the available versions `doctl kubernetes options versions`. (**Note:** A cluster may only be upgraded to newer versions in-place. If the version is decreased, a new resource will be created.)

        The **node_pool** object supports the following:

          * `actual_node_count` (`pulumi.Input[float]`)
          * `auto_scale` (`pulumi.Input[bool]`)
          * `id` (`pulumi.Input[str]`) - A unique ID that can be used to identify and reference a Kubernetes cluster.
          * `labels` (`pulumi.Input[dict]`)
          * `max_nodes` (`pulumi.Input[float]`)
          * `min_nodes` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - A name for the Kubernetes cluster.
          * `node_count` (`pulumi.Input[float]`)
          * `nodes` (`pulumi.Input[list]`)
            * `created_at` (`pulumi.Input[str]`) - The date and time when the Kubernetes cluster was created.
            * `droplet_id` (`pulumi.Input[str]`)
            * `id` (`pulumi.Input[str]`) - A unique ID that can be used to identify and reference a Kubernetes cluster.
            * `name` (`pulumi.Input[str]`) - A name for the Kubernetes cluster.
            * `status` (`pulumi.Input[str]`) - A string indicating the current status of the cluster. Potential values include running, provisioning, and errored.
            * `updated_at` (`pulumi.Input[str]`) - The date and time when the Kubernetes cluster was last updated.
              * `kube_config.0` - A representation of the Kubernetes cluster's kubeconfig with the following attributes:
              - `raw_config` - The full contents of the Kubernetes cluster's kubeconfig file.
              - `host` - The URL of the API server on the Kubernetes master node.
              - `cluster_ca_certificate` - The base64 encoded public certificate for the cluster's certificate authority.
              - `token` - The DigitalOcean API access token used by clients to access the cluster.
              - `client_key` - The base64 encoded private key used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
              - `client_certificate` - The base64 encoded public certificate used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
              - `expires_at` - The date and time when the credentials will expire and need to be regenerated.

          * `size` (`pulumi.Input[str]`)
          * `tags` (`pulumi.Input[list]`) - A list of tag names to be applied to the Kubernetes cluster.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['name'] = name
            if node_pool is None:
                raise TypeError("Missing required property 'node_pool'")
            __props__['node_pool'] = node_pool
            if region is None:
                raise TypeError("Missing required property 'region'")
            __props__['region'] = region
            __props__['tags'] = tags
            if version is None:
                raise TypeError("Missing required property 'version'")
            __props__['version'] = version
            __props__['cluster_subnet'] = None
            __props__['created_at'] = None
            __props__['endpoint'] = None
            __props__['ipv4_address'] = None
            __props__['kube_configs'] = None
            __props__['service_subnet'] = None
            __props__['status'] = None
            __props__['updated_at'] = None
        super(KubernetesCluster, __self__).__init__(
            'digitalocean:index/kubernetesCluster:KubernetesCluster',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, cluster_subnet=None, created_at=None, endpoint=None, ipv4_address=None, kube_configs=None, name=None, node_pool=None, region=None, service_subnet=None, status=None, tags=None, updated_at=None, version=None):
        """
        Get an existing KubernetesCluster resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] cluster_subnet: The range of IP addresses in the overlay network of the Kubernetes cluster.
        :param pulumi.Input[str] created_at: The date and time when the Kubernetes cluster was created.
        :param pulumi.Input[str] endpoint: The base URL of the API server on the Kubernetes master node.
        :param pulumi.Input[str] ipv4_address: The public IPv4 address of the Kubernetes master node.
        :param pulumi.Input[str] name: A name for the Kubernetes cluster.
        :param pulumi.Input[dict] node_pool: A block representing the cluster's default node pool. Additional node pools may be added to the cluster using the `.KubernetesNodePool` resource. The following arguments may be specified:
               - `name` - (Required) A name for the node pool.
               - `size` - (Required) The slug identifier for the type of Droplet to be used as workers in the node pool.
               - `node_count` - (Optional) The number of Droplet instances in the node pool. If auto-scaling is enabled, this should only be set if the desired result is to explicitly reset the number of nodes to this value. If auto-scaling is enabled, and the node count is outside of the given min/max range, it will use the min nodes value.
               - `auto_scale` - (Optional) Enable auto-scaling of the number of nodes in the node pool within the given min/max range.
               - `min_nodes` - (Optional) If auto-scaling is enabled, this represents the minimum number of nodes that the node pool can be scaled down to.
               - `max_nodes` - (Optional) If auto-scaling is enabled, this represents the maximum number of nodes that the node pool can be scaled up to.
               - `tags` - (Optional) A list of tag names to be applied to the Kubernetes cluster.
               - `labels` - (Optional) A map of key/value pairs to apply to nodes in the pool. The labels are exposed in the Kubernetes API as labels in the metadata of the corresponding [Node resources](https://kubernetes.io/docs/concepts/architecture/nodes/).
        :param pulumi.Input[str] region: The slug identifier for the region where the Kubernetes cluster will be created.
        :param pulumi.Input[str] service_subnet: The range of assignable IP addresses for services running in the Kubernetes cluster.
        :param pulumi.Input[str] status: A string indicating the current status of the cluster. Potential values include running, provisioning, and errored.
        :param pulumi.Input[list] tags: A list of tag names to be applied to the Kubernetes cluster.
        :param pulumi.Input[str] updated_at: The date and time when the Kubernetes cluster was last updated.
               * `kube_config.0` - A representation of the Kubernetes cluster's kubeconfig with the following attributes:
               - `raw_config` - The full contents of the Kubernetes cluster's kubeconfig file.
               - `host` - The URL of the API server on the Kubernetes master node.
               - `cluster_ca_certificate` - The base64 encoded public certificate for the cluster's certificate authority.
               - `token` - The DigitalOcean API access token used by clients to access the cluster.
               - `client_key` - The base64 encoded private key used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
               - `client_certificate` - The base64 encoded public certificate used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
               - `expires_at` - The date and time when the credentials will expire and need to be regenerated.
        :param pulumi.Input[str] version: The slug identifier for the version of Kubernetes used for the cluster. Use [doctl](https://github.com/digitalocean/doctl) to find the available versions `doctl kubernetes options versions`. (**Note:** A cluster may only be upgraded to newer versions in-place. If the version is decreased, a new resource will be created.)

        The **kube_configs** object supports the following:

          * `clientCertificate` (`pulumi.Input[str]`)
          * `clientKey` (`pulumi.Input[str]`)
          * `clusterCaCertificate` (`pulumi.Input[str]`)
          * `expiresAt` (`pulumi.Input[str]`)
          * `host` (`pulumi.Input[str]`)
          * `rawConfig` (`pulumi.Input[str]`)
          * `token` (`pulumi.Input[str]`)

        The **node_pool** object supports the following:

          * `actual_node_count` (`pulumi.Input[float]`)
          * `auto_scale` (`pulumi.Input[bool]`)
          * `id` (`pulumi.Input[str]`) - A unique ID that can be used to identify and reference a Kubernetes cluster.
          * `labels` (`pulumi.Input[dict]`)
          * `max_nodes` (`pulumi.Input[float]`)
          * `min_nodes` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - A name for the Kubernetes cluster.
          * `node_count` (`pulumi.Input[float]`)
          * `nodes` (`pulumi.Input[list]`)
            * `created_at` (`pulumi.Input[str]`) - The date and time when the Kubernetes cluster was created.
            * `droplet_id` (`pulumi.Input[str]`)
            * `id` (`pulumi.Input[str]`) - A unique ID that can be used to identify and reference a Kubernetes cluster.
            * `name` (`pulumi.Input[str]`) - A name for the Kubernetes cluster.
            * `status` (`pulumi.Input[str]`) - A string indicating the current status of the cluster. Potential values include running, provisioning, and errored.
            * `updated_at` (`pulumi.Input[str]`) - The date and time when the Kubernetes cluster was last updated.
              * `kube_config.0` - A representation of the Kubernetes cluster's kubeconfig with the following attributes:
              - `raw_config` - The full contents of the Kubernetes cluster's kubeconfig file.
              - `host` - The URL of the API server on the Kubernetes master node.
              - `cluster_ca_certificate` - The base64 encoded public certificate for the cluster's certificate authority.
              - `token` - The DigitalOcean API access token used by clients to access the cluster.
              - `client_key` - The base64 encoded private key used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
              - `client_certificate` - The base64 encoded public certificate used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
              - `expires_at` - The date and time when the credentials will expire and need to be regenerated.

          * `size` (`pulumi.Input[str]`)
          * `tags` (`pulumi.Input[list]`) - A list of tag names to be applied to the Kubernetes cluster.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["cluster_subnet"] = cluster_subnet
        __props__["created_at"] = created_at
        __props__["endpoint"] = endpoint
        __props__["ipv4_address"] = ipv4_address
        __props__["kube_configs"] = kube_configs
        __props__["name"] = name
        __props__["node_pool"] = node_pool
        __props__["region"] = region
        __props__["service_subnet"] = service_subnet
        __props__["status"] = status
        __props__["tags"] = tags
        __props__["updated_at"] = updated_at
        __props__["version"] = version
        return KubernetesCluster(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class SshKey(pulumi.CustomResource):
    fingerprint: pulumi.Output[str]
    """
    The fingerprint of the SSH key
    """
    name: pulumi.Output[str]
    """
    The name of the SSH key for identification
    """
    public_key: pulumi.Output[str]
    """
    The public key. If this is a file, it
    can be read using the file interpolation function
    """
    def __init__(__self__, resource_name, opts=None, name=None, public_key=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a DigitalOcean SSH key resource to allow you to manage SSH
        keys for Droplet access. Keys created with this resource
        can be referenced in your Droplet configuration via their ID or
        fingerprint.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/r/ssh_key.html.markdown.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] name: The name of the SSH key for identification
        :param pulumi.Input[str] public_key: The public key. If this is a file, it
               can be read using the file interpolation function
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['name'] = name
            if public_key is None:
                raise TypeError("Missing required property 'public_key'")
            __props__['public_key'] = public_key
            __props__['fingerprint'] = None
        super(SshKey, __self__).__init__(
            'digitalocean:index/sshKey:SshKey',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, fingerprint=None, name=None, public_key=None):
        """
        Get an existing SshKey resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] fingerprint: The fingerprint of the SSH key
        :param pulumi.Input[str] name: The name of the SSH key for identification
        :param pulumi.Input[str] public_key: The public key. If this is a file, it
               can be read using the file interpolation function
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["fingerprint"] = fingerprint
        __props__["name"] = name
        __props__["public_key"] = public_key
        return SshKey(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class Project(pulumi.CustomResource):
    created_at: pulumi.Output[str]
    """
    the date and time when the project was created, (ISO8601)
    """
    description: pulumi.Output[str]
    """
    the description of the project
    """
    environment: pulumi.Output[str]
    """
    the environment of the project's resources. The possible values are: `Development`, `Staging`, `Production`)
    """
    is_default: pulumi.Output[bool]
    name: pulumi.Output[str]
    """
    The name of the Project
    """
    owner_id: pulumi.Output[float]
    """
    the id of the project owner.
    """
    owner_uuid: pulumi.Output[str]
    """
    the unique universal identifier of the project owner.
    """
    purpose: pulumi.Output[str]
    """
    the purpose of the project, (Default: "Web Application")
    """
    resources: pulumi.Output[list]
    """
    a list of uniform resource names (URNs) for the resources associated with the project
    """
    updated_at: pulumi.Output[str]
    """
    the date and time when the project was last updated, (ISO8601)
    """
    def __init__(__self__, resource_name, opts=None, description=None, environment=None, name=None, purpose=None, resources=None, __props__=None, __name__=None, __opts__=None):
        """
        Create a Project resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: the description of the project
        :param pulumi.Input[str] environment: the environment of the project's resources. The possible values are: `Development`, `Staging`, `Production`)
        :param pulumi.Input[str] name: The name of the Project
        :param pulumi.Input[str] purpose: the purpose of the project, (Default: "Web Application")
        :param pulumi.Input[list] resources: a list of uniform resource names (URNs) for the resources associated with the project
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['description'] = description
            __props__['environment'] = environment
            __props__['name'] = name
            __props__['purpose'] = purpose
            __props__['resources'] = resources
            __props__['created_at'] = None
            __props__['is_default'] = None
            __props__['owner_id'] = None
            __props__['owner_uuid'] = None
            __props__['updated_at'] = None
        super(Project, __self__).__init__(
            'digitalocean:index/project:Project',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, created_at=None, description=None, environment=None, is_default=None, name=None, owner_id=None, owner_uuid=None, purpose=None, resources=None, updated_at=None):
        """
        Get an existing Project resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] created_at: the date and time when the project was created, (ISO8601)
        :param pulumi.Input[str] description: the description of the project
        :param pulumi.Input[str] environment: the environment of the project's resources. The possible values are: `Development`, `Staging`, `Production`)
        :param pulumi.Input[str] name: The name of the Project
        :param pulumi.Input[float] owner_id: the id of the project owner.
        :param pulumi.Input[str] owner_uuid: the unique universal identifier of the project owner.
        :param pulumi.Input[str] purpose: the purpose of the project, (Default: "Web Application")
        :param pulumi.Input[list] resources: a list of uniform resource names (URNs) for the resources associated with the project
        :param pulumi.Input[str] updated_at: the date and time when the project was last updated, (ISO8601)
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["created_at"] = created_at
        __props__["description"] = description
        __props__["environment"] = environment
        __props__["is_default"] = is_default
        __props__["name"] = name
        __props__["owner_id"] = owner_id
        __props__["owner_uuid"] = owner_uuid
        __props__["purpose"] = purpose
        __props__["resources"] = resources
        __props__["updated_at"] = updated_at
        return Project(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class GetSizesResult:
    """
    A collection of values returned by getSizes.
    """
    def __init__(__self__, filters=None, id=None, sizes=None, sorts=None):
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        __self__.filters = filters
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
        if sizes and not isinstance(sizes, list):
            raise TypeError("Expected argument 'sizes' to be a list")
        __self__.sizes = sizes
        if sorts and not isinstance(sorts, list):
            raise TypeError("Expected argument 'sorts' to be a list")
        __self__.sorts = sorts
class AwaitableGetSizesResult(GetSizesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetSizesResult(
            filters=self.filters,
            id=self.id,
            sizes=self.sizes,
            sorts=self.sorts)

def get_sizes(filters=None,sorts=None,opts=None):
    """
    Retrieves information about the Droplet sizes that DigitalOcean supports, with
    the ability to filter and sort the results. If no filters are specified, all sizes
    will be returned.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/d/sizes.html.md.


    :param list filters: Filter the results.
           The `filter` block is documented below.
    :param list sorts: Sort the results.
           The `sort` block is documented below.

    The **filters** object supports the following:

      * `key` (`str`) - Filter the sizes by this key. This may be one of `slug`,
        `regions`, `memory`, `vcpus`, `disk`, `transfer`, `price_monthly`,
        `price_hourly`, or `available`.
      * `values` (`list`) - Only retrieves images which keys has value that matches
        one of the values provided here.

    The **sorts** object supports the following:

      * `direction` (`str`) - The sort direction. This may be either `asc` or `desc`.
      * `key` (`str`) - Sort the sizes by this key. This may be one of `slug`,
        `memory`, `vcpus`, `disk`, `transfer`, `price_monthly`, or `price_hourly`.
    """
    __args__ = dict()


    __args__['filters'] = filters
    __args__['sorts'] = sorts
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getSizes:getSizes', __args__, opts=opts).value

    return AwaitableGetSizesResult(
        filters=__ret__.get('filters'),
        id=__ret__.get('id'),
        sizes=__ret__.get('sizes'),
        sorts=__ret__.get('sorts'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class GetImagesResult:
    """
    A collection of values returned by getImages.
    """
    def __init__(__self__, filters=None, id=None, images=None, sorts=None):
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        __self__.filters = filters
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
        if images and not isinstance(images, list):
            raise TypeError("Expected argument 'images' to be a list")
        __self__.images = images
        """
        A set of images satisfying any `filter` and `sort` criteria. Each image has the following attributes:  
        - `slug`: Unique text identifier of the image.
        - `id`: The ID of the image.
        - `name`: The name of the image.
        - `type`: Type of the image.
        - `distribution` - The name of the distribution of the OS of the image.
        - `min_disk_size`: The minimum 'disk' required for the image.
        - `size_gigabytes`: The size of the image in GB.
        - `private` - Is image a public image or not. Public images represent
        Linux distributions or One-Click Applications, while non-public images represent
        snapshots and backups and are only available within your account.
        - `regions`: A set of the regions that the image is available in.
        - `tags`: A set of tags applied to the image
        - `created`: When the image was created
        - `status`: Current status of the image
        - `error_message`: Any applicable error message pertaining to the image
        - `image` - The id of the image (legacy parameter).
        """
        if sorts and not isinstance(sorts, list):
            raise TypeError("Expected argument 'sorts' to be a list")
        __self__.sorts = sorts
class AwaitableGetImagesResult(GetImagesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetImagesResult(
            filters=self.filters,
            id=self.id,
            images=self.images,
            sorts=self.sorts)

def get_images(filters=None,sorts=None,opts=None):
    """
    Use this data source to access information about an existing resource.

    :param list filters: Filter the results.
           The `filter` block is documented below.
    :param list sorts: Sort the results.
           The `sort` block is documented below.

    The **filters** object supports the following:

      * `key` (`str`) - Filter the images by this key. This may be one of `distribution`, `error_message`,
        `id`, `image`, `min_disk_size`, `name`, `private`, `regions`, `size_gigabytes`, `slug`, `status`,
        `tags`, or `type`.
      * `values` (`list`) - A list of values to match against the `key` field. Only retrieves images
        where the `key` field takes on one or more of the values provided here.

    The **sorts** object supports the following:

      * `direction` (`str`) - The sort direction. This may be either `asc` or `desc`.
      * `key` (`str`) - Sort the images by this key. This may be one of `distribution`, `error_message`, `id`,
        `image`, `min_disk_size`, `name`, `private`, `size_gigabytes`, `slug`, `status`, or `type`.
    """
    __args__ = dict()


    __args__['filters'] = filters
    __args__['sorts'] = sorts
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getImages:getImages', __args__, opts=opts).value

    return AwaitableGetImagesResult(
        filters=__ret__.get('filters'),
        id=__ret__.get('id'),
        images=__ret__.get('images'),
        sorts=__ret__.get('sorts'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class DnsRecord(pulumi.CustomResource):
    domain: pulumi.Output[str]
    """
    The domain to add the record to.
    """
    flags: pulumi.Output[float]
    """
    The flags of the record. Only valid when type is `CAA`. Must be between 0 and 255.
    """
    fqdn: pulumi.Output[str]
    """
    The FQDN of the record
    """
    name: pulumi.Output[str]
    """
    The name of the record. Use `@` for records on domain's name itself.
    """
    port: pulumi.Output[float]
    """
    The port of the record. Only valid when type is `SRV`.  Must be between 1 and 65535.
    """
    priority: pulumi.Output[float]
    """
    The priority of the record. Only valid when type is `MX` or `SRV`. Must be between 0 and 65535.
    """
    tag: pulumi.Output[str]
    """
    The tag of the record. Only valid when type is `CAA`. Must be one of `issue`, `issuewild`, or `iodef`.
    """
    ttl: pulumi.Output[float]
    """
    The time to live for the record, in seconds. Must be at least 0.
    """
    type: pulumi.Output[str]
    """
    The type of record. Must be one of `A`, `AAAA`, `CAA`, `CNAME`, `MX`, `NS`, `TXT`, or `SRV`.
    """
    value: pulumi.Output[str]
    """
    The value of the record.
    """
    weight: pulumi.Output[float]
    """
    The weight of the record. Only valid when type is `SRV`.  Must be between 0 and 65535.
    """
    def __init__(__self__, resource_name, opts=None, domain=None, flags=None, name=None, port=None, priority=None, tag=None, ttl=None, type=None, value=None, weight=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a DigitalOcean DNS record resource.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/r/record.html.markdown.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] domain: The domain to add the record to.
        :param pulumi.Input[float] flags: The flags of the record. Only valid when type is `CAA`. Must be between 0 and 255.
        :param pulumi.Input[str] name: The name of the record. Use `@` for records on domain's name itself.
        :param pulumi.Input[float] port: The port of the record. Only valid when type is `SRV`.  Must be between 1 and 65535.
        :param pulumi.Input[float] priority: The priority of the record. Only valid when type is `MX` or `SRV`. Must be between 0 and 65535.
        :param pulumi.Input[str] tag: The tag of the record. Only valid when type is `CAA`. Must be one of `issue`, `issuewild`, or `iodef`.
        :param pulumi.Input[float] ttl: The time to live for the record, in seconds. Must be at least 0.
        :param pulumi.Input[str] type: The type of record. Must be one of `A`, `AAAA`, `CAA`, `CNAME`, `MX`, `NS`, `TXT`, or `SRV`.
        :param pulumi.Input[str] value: The value of the record.
        :param pulumi.Input[float] weight: The weight of the record. Only valid when type is `SRV`.  Must be between 0 and 65535.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if domain is None:
                raise TypeError("Missing required property 'domain'")
            __props__['domain'] = domain
            __props__['flags'] = flags
            __props__['name'] = name
            __props__['port'] = port
            __props__['priority'] = priority
            __props__['tag'] = tag
            __props__['ttl'] = ttl
            if type is None:
                raise TypeError("Missing required property 'type'")
            __props__['type'] = type
            if value is None:
                raise TypeError("Missing required property 'value'")
            __props__['value'] = value
            __props__['weight'] = weight
            __props__['fqdn'] = None
        super(DnsRecord, __self__).__init__(
            'digitalocean:index/dnsRecord:DnsRecord',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, domain=None, flags=None, fqdn=None, name=None, port=None, priority=None, tag=None, ttl=None, type=None, value=None, weight=None):
        """
        Get an existing DnsRecord resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] domain: The domain to add the record to.
        :param pulumi.Input[float] flags: The flags of the record. Only valid when type is `CAA`. Must be between 0 and 255.
        :param pulumi.Input[str] fqdn: The FQDN of the record
        :param pulumi.Input[str] name: The name of the record. Use `@` for records on domain's name itself.
        :param pulumi.Input[float] port: The port of the record. Only valid when type is `SRV`.  Must be between 1 and 65535.
        :param pulumi.Input[float] priority: The priority of the record. Only valid when type is `MX` or `SRV`. Must be between 0 and 65535.
        :param pulumi.Input[str] tag: The tag of the record. Only valid when type is `CAA`. Must be one of `issue`, `issuewild`, or `iodef`.
        :param pulumi.Input[float] ttl: The time to live for the record, in seconds. Must be at least 0.
        :param pulumi.Input[str] type: The type of record. Must be one of `A`, `AAAA`, `CAA`, `CNAME`, `MX`, `NS`, `TXT`, or `SRV`.
        :param pulumi.Input[str] value: The value of the record.
        :param pulumi.Input[float] weight: The weight of the record. Only valid when type is `SRV`.  Must be between 0 and 65535.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["domain"] = domain
        __props__["flags"] = flags
        __props__["fqdn"] = fqdn
        __props__["name"] = name
        __props__["port"] = port
        __props__["priority"] = priority
        __props__["tag"] = tag
        __props__["ttl"] = ttl
        __props__["type"] = type
        __props__["value"] = value
        __props__["weight"] = weight
        return DnsRecord(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

_SNAKE_TO_CAMEL_CASE_TABLE = {
    "actual_node_count": "actualNodeCount",
    "api_endpoint": "apiEndpoint",
    "auto_scale": "autoScale",
    "bucket_domain_name": "bucketDomainName",
    "certificate_chain": "certificateChain",
    "certificate_id": "certificateId",
    "cluster_id": "clusterId",
    "cluster_subnet": "clusterSubnet",
    "cors_rules": "corsRules",
    "created_at": "createdAt",
    "custom_domain": "customDomain",
    "db_name": "dbName",
    "droplet_id": "dropletId",
    "droplet_ids": "dropletIds",
    "droplet_tag": "dropletTag",
    "enable_proxy_protocol": "enableProxyProtocol",
    "eviction_policy": "evictionPolicy",
    "filesystem_label": "filesystemLabel",
    "filesystem_type": "filesystemType",
    "force_destroy": "forceDestroy",
    "forwarding_rules": "forwardingRules",
    "inbound_rules": "inboundRules",
    "initial_filesystem_label": "initialFilesystemLabel",
    "initial_filesystem_type": "initialFilesystemType",
    "ip_address": "ipAddress",
    "ipv4_address": "ipv4Address",
    "ipv4_address_private": "ipv4AddressPrivate",
    "ipv6_address": "ipv6Address",
    "is_default": "isDefault",
    "kube_configs": "kubeConfigs",
    "leaf_certificate": "leafCertificate",
    "maintenance_windows": "maintenanceWindows",
    "max_nodes": "maxNodes",
    "min_disk_size": "minDiskSize",
    "min_nodes": "minNodes",
    "mysql_auth_plugin": "mysqlAuthPlugin",
    "node_count": "nodeCount",
    "node_pool": "nodePool",
    "not_after": "notAfter",
    "outbound_rules": "outboundRules",
    "owner_id": "ownerId",
    "owner_uuid": "ownerUuid",
    "pending_changes": "pendingChanges",
    "price_hourly": "priceHourly",
    "price_monthly": "priceMonthly",
    "private_host": "privateHost",
    "private_key": "privateKey",
    "private_networking": "privateNetworking",
    "private_uri": "privateUri",
    "public_key": "publicKey",
    "redirect_http_to_https": "redirectHttpToHttps",
    "resize_disk": "resizeDisk",
    "service_subnet": "serviceSubnet",
    "sha1_fingerprint": "sha1Fingerprint",
    "snapshot_id": "snapshotId",
    "spaces_access_id": "spacesAccessId",
    "spaces_secret_key": "spacesSecretKey",
    "sql_mode": "sqlMode",
    "ssh_keys": "sshKeys",
    "sticky_sessions": "stickySessions",
    "updated_at": "updatedAt",
    "user_data": "userData",
    "volume_id": "volumeId",
    "volume_ids": "volumeIds",
}

_CAMEL_TO_SNAKE_CASE_TABLE = {
    "actualNodeCount": "actual_node_count",
    "apiEndpoint": "api_endpoint",
    "autoScale": "auto_scale",
    "bucketDomainName": "bucket_domain_name",
    "certificateChain": "certificate_chain",
    "certificateId": "certificate_id",
    "clusterId": "cluster_id",
    "clusterSubnet": "cluster_subnet",
    "corsRules": "cors_rules",
    "createdAt": "created_at",
    "customDomain": "custom_domain",
    "dbName": "db_name",
    "dropletId": "droplet_id",
    "dropletIds": "droplet_ids",
    "dropletTag": "droplet_tag",
    "enableProxyProtocol": "enable_proxy_protocol",
    "evictionPolicy": "eviction_policy",
    "filesystemLabel": "filesystem_label",
    "filesystemType": "filesystem_type",
    "forceDestroy": "force_destroy",
    "forwardingRules": "forwarding_rules",
    "inboundRules": "inbound_rules",
    "initialFilesystemLabel": "initial_filesystem_label",
    "initialFilesystemType": "initial_filesystem_type",
    "ipAddress": "ip_address",
    "ipv4Address": "ipv4_address",
    "ipv4AddressPrivate": "ipv4_address_private",
    "ipv6Address": "ipv6_address",
    "isDefault": "is_default",
    "kubeConfigs": "kube_configs",
    "leafCertificate": "leaf_certificate",
    "maintenanceWindows": "maintenance_windows",
    "maxNodes": "max_nodes",
    "minDiskSize": "min_disk_size",
    "minNodes": "min_nodes",
    "mysqlAuthPlugin": "mysql_auth_plugin",
    "nodeCount": "node_count",
    "nodePool": "node_pool",
    "notAfter": "not_after",
    "outboundRules": "outbound_rules",
    "ownerId": "owner_id",
    "ownerUuid": "owner_uuid",
    "pendingChanges": "pending_changes",
    "priceHourly": "price_hourly",
    "priceMonthly": "price_monthly",
    "privateHost": "private_host",
    "privateKey": "private_key",
    "privateNetworking": "private_networking",
    "privateUri": "private_uri",
    "publicKey": "public_key",
    "redirectHttpToHttps": "redirect_http_to_https",
    "resizeDisk": "resize_disk",
    "serviceSubnet": "service_subnet",
    "sha1Fingerprint": "sha1_fingerprint",
    "snapshotId": "snapshot_id",
    "spacesAccessId": "spaces_access_id",
    "spacesSecretKey": "spaces_secret_key",
    "sqlMode": "sql_mode",
    "sshKeys": "ssh_keys",
    "stickySessions": "sticky_sessions",
    "updatedAt": "updated_at",
    "userData": "user_data",
    "volumeId": "volume_id",
    "volumeIds": "volume_ids",
}



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class GetKubernetesClusterResult:
    """
    A collection of values returned by getKubernetesCluster.
    """
    def __init__(__self__, cluster_subnet=None, created_at=None, endpoint=None, id=None, ipv4_address=None, kube_configs=None, name=None, node_pools=None, region=None, service_subnet=None, status=None, tags=None, updated_at=None, version=None):
        if cluster_subnet and not isinstance(cluster_subnet, str):
            raise TypeError("Expected argument 'cluster_subnet' to be a str")
        __self__.cluster_subnet = cluster_subnet
        """
        The range of IP addresses in the overlay network of the Kubernetes cluster.
        """
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        __self__.created_at = created_at
        """
        The date and time when the Kubernetes cluster was created.
        """
        if endpoint and not isinstance(endpoint, str):
            raise TypeError("Expected argument 'endpoint' to be a str")
        __self__.endpoint = endpoint
        """
        The base URL of the API server on the Kubernetes master node.
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
        if ipv4_address and not isinstance(ipv4_address, str):
            raise TypeError("Expected argument 'ipv4_address' to be a str")
        __self__.ipv4_address = ipv4_address
        """
        The public IPv4 address of the Kubernetes master node.
        """
        if kube_configs and not isinstance(kube_configs, list):
            raise TypeError("Expected argument 'kube_configs' to be a list")
        __self__.kube_configs = kube_configs
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        __self__.name = name
        if node_pools and not isinstance(node_pools, list):
            raise TypeError("Expected argument 'node_pools' to be a list")
        __self__.node_pools = node_pools
        """
        A list of node pools associated with the cluster. Each node pool exports the following attributes:
        - `id` -  The unique ID that can be used to identify and reference the node pool.
        - `name` - The name of the node pool.
        - `size` - The slug identifier for the type of Droplet used as workers in the node pool.
        - `node_count` - The number of Droplet instances in the node pool.
        - `actual_node_count` - The actual number of nodes in the node pool, which is especially useful when auto-scaling is enabled.
        - `auto_scale` - A boolean indicating whether auto-scaling is enabled on the node pool.
        - `min_nodes` - If auto-scaling is enabled, this represents the minimum number of nodes that the node pool can be scaled down to.
        - `max_nodes` - If auto-scaling is enabled, this represents the maximum number of nodes that the node pool can be scaled up to.
        - `tags` - A list of tag names applied to the node pool.
        - `labels` - A map of key/value pairs applied to nodes in the pool. The labels are exposed in the Kubernetes API as labels in the metadata of the corresponding [Node resources](https://kubernetes.io/docs/concepts/architecture/nodes/).
        - `nodes` - A list of nodes in the pool. Each node exports the following attributes:
        + `id` -  A unique ID that can be used to identify and reference the node.
        + `name` - The auto-generated name for the node.
        + `status` -  A string indicating the current status of the individual node.
        + `created_at` - The date and time when the node was created.
        + `updated_at` - The date and time when the node was last updated.
        """
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        __self__.region = region
        """
        The slug identifier for the region where the Kubernetes cluster is located.
        """
        if service_subnet and not isinstance(service_subnet, str):
            raise TypeError("Expected argument 'service_subnet' to be a str")
        __self__.service_subnet = service_subnet
        """
        The range of assignable IP addresses for services running in the Kubernetes cluster.
        """
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        __self__.status = status
        """
        A string indicating the current status of the cluster. Potential values include running, provisioning, and errored.
        """
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        __self__.tags = tags
        """
        A list of tag names to be applied to the Kubernetes cluster.
        """
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        __self__.updated_at = updated_at
        """
        The date and time when the Kubernetes cluster was last updated.
        * `kube_config.0` - A representation of the Kubernetes cluster's kubeconfig with the following attributes:
        - `raw_config` - The full contents of the Kubernetes cluster's kubeconfig file.
        - `host` - The URL of the API server on the Kubernetes master node.
        - `cluster_ca_certificate` - The base64 encoded public certificate for the cluster's certificate authority.
        - `token` - The DigitalOcean API access token used by clients to access the cluster.
        - `client_key` - The base64 encoded private key used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
        - `client_certificate` - The base64 encoded public certificate used by clients to access the cluster. Only available if token authentication is not supported on your cluster.
        - `expires_at` - The date and time when the credentials will expire and need to be regenerated.
        """
        if version and not isinstance(version, str):
            raise TypeError("Expected argument 'version' to be a str")
        __self__.version = version
        """
        The slug identifier for the version of Kubernetes used for the cluster.
        """
class AwaitableGetKubernetesClusterResult(GetKubernetesClusterResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetKubernetesClusterResult(
            cluster_subnet=self.cluster_subnet,
            created_at=self.created_at,
            endpoint=self.endpoint,
            id=self.id,
            ipv4_address=self.ipv4_address,
            kube_configs=self.kube_configs,
            name=self.name,
            node_pools=self.node_pools,
            region=self.region,
            service_subnet=self.service_subnet,
            status=self.status,
            tags=self.tags,
            updated_at=self.updated_at,
            version=self.version)

def get_kubernetes_cluster(name=None,tags=None,opts=None):
    """
    Use this data source to access information about an existing resource.

    :param str name: The name of Kubernetes cluster.
    :param list tags: A list of tag names to be applied to the Kubernetes cluster.
    """
    __args__ = dict()


    __args__['name'] = name
    __args__['tags'] = tags
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getKubernetesCluster:getKubernetesCluster', __args__, opts=opts).value

    return AwaitableGetKubernetesClusterResult(
        cluster_subnet=__ret__.get('clusterSubnet'),
        created_at=__ret__.get('createdAt'),
        endpoint=__ret__.get('endpoint'),
        id=__ret__.get('id'),
        ipv4_address=__ret__.get('ipv4Address'),
        kube_configs=__ret__.get('kubeConfigs'),
        name=__ret__.get('name'),
        node_pools=__ret__.get('nodePools'),
        region=__ret__.get('region'),
        service_subnet=__ret__.get('serviceSubnet'),
        status=__ret__.get('status'),
        tags=__ret__.get('tags'),
        updated_at=__ret__.get('updatedAt'),
        version=__ret__.get('version'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class GetRegionResult:
    """
    A collection of values returned by getRegion.
    """
    def __init__(__self__, available=None, features=None, id=None, name=None, sizes=None, slug=None):
        if available and not isinstance(available, bool):
            raise TypeError("Expected argument 'available' to be a bool")
        __self__.available = available
        """
        A boolean value that represents whether new Droplets can be created in this region.
        """
        if features and not isinstance(features, list):
            raise TypeError("Expected argument 'features' to be a list")
        __self__.features = features
        """
        A set of features available in this region.
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        __self__.name = name
        """
        The display name of the region.
        """
        if sizes and not isinstance(sizes, list):
            raise TypeError("Expected argument 'sizes' to be a list")
        __self__.sizes = sizes
        """
        A set of identifying slugs for the Droplet sizes available in this region.
        """
        if slug and not isinstance(slug, str):
            raise TypeError("Expected argument 'slug' to be a str")
        __self__.slug = slug
        """
        A human-readable string that is used as a unique identifier for each region.
        """
class AwaitableGetRegionResult(GetRegionResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetRegionResult(
            available=self.available,
            features=self.features,
            id=self.id,
            name=self.name,
            sizes=self.sizes,
            slug=self.slug)

def get_region(slug=None,opts=None):
    """
    Get information on a single DigitalOcean region. This is useful to find out 
    what Droplet sizes and features are supported within a region.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/d/region.html.md.


    :param str slug: A human-readable string that is used as a unique identifier for each region.
    """
    __args__ = dict()


    __args__['slug'] = slug
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getRegion:getRegion', __args__, opts=opts).value

    return AwaitableGetRegionResult(
        available=__ret__.get('available'),
        features=__ret__.get('features'),
        id=__ret__.get('id'),
        name=__ret__.get('name'),
        sizes=__ret__.get('sizes'),
        slug=__ret__.get('slug'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class LoadBalancer(pulumi.CustomResource):
    algorithm: pulumi.Output[str]
    """
    The load balancing algorithm used to determine
    which backend Droplet will be selected by a client. It must be either `round_robin`
    or `least_connections`. The default value is `round_robin`.
    """
    droplet_ids: pulumi.Output[list]
    """
    A list of the IDs of each droplet to be attached to the Load Balancer.
    """
    droplet_tag: pulumi.Output[str]
    """
    The name of a Droplet tag corresponding to Droplets to be assigned to the Load Balancer.
    """
    enable_proxy_protocol: pulumi.Output[bool]
    """
    A boolean value indicating whether PROXY
    Protocol should be used to pass information from connecting client requests to
    the backend service. Default value is `false`.
    """
    forwarding_rules: pulumi.Output[list]
    """
    A list of `forwarding_rule` to be assigned to the
    Load Balancer. The `forwarding_rule` block is documented below.

      * `certificate_id` (`str`) - The ID of the TLS certificate to be used for SSL termination.
      * `entryPort` (`float`) - An integer representing the port on which the Load Balancer instance will listen.
      * `entryProtocol` (`str`) - The protocol used for traffic to the Load Balancer. The possible values are: `http`, `https`, `http2` or `tcp`.
      * `targetPort` (`float`) - An integer representing the port on the backend Droplets to which the Load Balancer will send traffic.
      * `targetProtocol` (`str`) - The protocol used for traffic from the Load Balancer to the backend Droplets. The possible values are: `http`, `https`, `http2` or `tcp`.
      * `tlsPassthrough` (`bool`) - A boolean value indicating whether SSL encrypted traffic will be passed through to the backend Droplets. The default value is `false`.
    """
    healthcheck: pulumi.Output[dict]
    """
    A `healthcheck` block to be assigned to the
    Load Balancer. The `healthcheck` block is documented below. Only 1 healthcheck is allowed.

      * `checkIntervalSeconds` (`float`) - The number of seconds between between two consecutive health checks. If not specified, the default value is `10`.
      * `healthyThreshold` (`float`) - The number of times a health check must pass for a backend Droplet to be marked "healthy" and be re-added to the pool. If not specified, the default value is `5`.
      * `path` (`str`) - The path on the backend Droplets to which the Load Balancer instance will send a request.
      * `port` (`float`) - An integer representing the port on the backend Droplets on which the health check will attempt a connection.
      * `protocol` (`str`) - The protocol used for health checks sent to the backend Droplets. The possible values are `http` or `tcp`.
      * `responseTimeoutSeconds` (`float`) - The number of seconds the Load Balancer instance will wait for a response until marking a health check as failed. If not specified, the default value is `5`.
      * `unhealthyThreshold` (`float`) - The number of times a health check must fail for a backend Droplet to be marked "unhealthy" and be removed from the pool. If not specified, the default value is `3`.
    """
    ip: pulumi.Output[str]
    name: pulumi.Output[str]
    """
    The Load Balancer name
    """
    redirect_http_to_https: pulumi.Output[bool]
    """
    A boolean value indicating whether
    HTTP requests to the Load Balancer on port 80 will be redirected to HTTPS on port 443.
    Default value is `false`.
    """
    region: pulumi.Output[str]
    """
    The region to start in
    """
    status: pulumi.Output[str]
    sticky_sessions: pulumi.Output[dict]
    """
    A `sticky_sessions` block to be assigned to the
    Load Balancer. The `sticky_sessions` block is documented below. Only 1 sticky_sessions block is allowed.

      * `cookieName` (`str`) - The name to be used for the cookie sent to the client. This attribute is required when using `cookies` for the sticky sessions type.
      * `cookieTtlSeconds` (`float`) - The number of seconds until the cookie set by the Load Balancer expires. This attribute is required when using `cookies` for the sticky sessions type.
      * `type` (`str`) - An attribute indicating how and if requests from a client will be persistently served by the same backend Droplet. The possible values are `cookies` or `none`. If not specified, the default value is `none`.
    """
    urn: pulumi.Output[str]
    """
    The uniform resource name for the Load Balancer
    """
    def __init__(__self__, resource_name, opts=None, algorithm=None, droplet_ids=None, droplet_tag=None, enable_proxy_protocol=None, forwarding_rules=None, healthcheck=None, name=None, redirect_http_to_https=None, region=None, sticky_sessions=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a DigitalOcean Load Balancer resource. This can be used to create,
        modify, and delete Load Balancers.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/r/loadbalancer.html.markdown.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] algorithm: The load balancing algorithm used to determine
               which backend Droplet will be selected by a client. It must be either `round_robin`
               or `least_connections`. The default value is `round_robin`.
        :param pulumi.Input[list] droplet_ids: A list of the IDs of each droplet to be attached to the Load Balancer.
        :param pulumi.Input[str] droplet_tag: The name of a Droplet tag corresponding to Droplets to be assigned to the Load Balancer.
        :param pulumi.Input[bool] enable_proxy_protocol: A boolean value indicating whether PROXY
               Protocol should be used to pass information from connecting client requests to
               the backend service. Default value is `false`.
        :param pulumi.Input[list] forwarding_rules: A list of `forwarding_rule` to be assigned to the
               Load Balancer. The `forwarding_rule` block is documented below.
        :param pulumi.Input[dict] healthcheck: A `healthcheck` block to be assigned to the
               Load Balancer. The `healthcheck` block is documented below. Only 1 healthcheck is allowed.
        :param pulumi.Input[str] name: The Load Balancer name
        :param pulumi.Input[bool] redirect_http_to_https: A boolean value indicating whether
               HTTP requests to the Load Balancer on port 80 will be redirected to HTTPS on port 443.
               Default value is `false`.
        :param pulumi.Input[str] region: The region to start in
        :param pulumi.Input[dict] sticky_sessions: A `sticky_sessions` block to be assigned to the
               Load Balancer. The `sticky_sessions` block is documented below. Only 1 sticky_sessions block is allowed.

        The **forwarding_rules** object supports the following:

          * `certificate_id` (`pulumi.Input[str]`) - The ID of the TLS certificate to be used for SSL termination.
          * `entryPort` (`pulumi.Input[float]`) - An integer representing the port on which the Load Balancer instance will listen.
          * `entryProtocol` (`pulumi.Input[str]`) - The protocol used for traffic to the Load Balancer. The possible values are: `http`, `https`, `http2` or `tcp`.
          * `targetPort` (`pulumi.Input[float]`) - An integer representing the port on the backend Droplets to which the Load Balancer will send traffic.
          * `targetProtocol` (`pulumi.Input[str]`) - The protocol used for traffic from the Load Balancer to the backend Droplets. The possible values are: `http`, `https`, `http2` or `tcp`.
          * `tlsPassthrough` (`pulumi.Input[bool]`) - A boolean value indicating whether SSL encrypted traffic will be passed through to the backend Droplets. The default value is `false`.

        The **healthcheck** object supports the following:

          * `checkIntervalSeconds` (`pulumi.Input[float]`) - The number of seconds between between two consecutive health checks. If not specified, the default value is `10`.
          * `healthyThreshold` (`pulumi.Input[float]`) - The number of times a health check must pass for a backend Droplet to be marked "healthy" and be re-added to the pool. If not specified, the default value is `5`.
          * `path` (`pulumi.Input[str]`) - The path on the backend Droplets to which the Load Balancer instance will send a request.
          * `port` (`pulumi.Input[float]`) - An integer representing the port on the backend Droplets on which the health check will attempt a connection.
          * `protocol` (`pulumi.Input[str]`) - The protocol used for health checks sent to the backend Droplets. The possible values are `http` or `tcp`.
          * `responseTimeoutSeconds` (`pulumi.Input[float]`) - The number of seconds the Load Balancer instance will wait for a response until marking a health check as failed. If not specified, the default value is `5`.
          * `unhealthyThreshold` (`pulumi.Input[float]`) - The number of times a health check must fail for a backend Droplet to be marked "unhealthy" and be removed from the pool. If not specified, the default value is `3`.

        The **sticky_sessions** object supports the following:

          * `cookieName` (`pulumi.Input[str]`) - The name to be used for the cookie sent to the client. This attribute is required when using `cookies` for the sticky sessions type.
          * `cookieTtlSeconds` (`pulumi.Input[float]`) - The number of seconds until the cookie set by the Load Balancer expires. This attribute is required when using `cookies` for the sticky sessions type.
          * `type` (`pulumi.Input[str]`) - An attribute indicating how and if requests from a client will be persistently served by the same backend Droplet. The possible values are `cookies` or `none`. If not specified, the default value is `none`.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['algorithm'] = algorithm
            __props__['droplet_ids'] = droplet_ids
            __props__['droplet_tag'] = droplet_tag
            __props__['enable_proxy_protocol'] = enable_proxy_protocol
            if forwarding_rules is None:
                raise TypeError("Missing required property 'forwarding_rules'")
            __props__['forwarding_rules'] = forwarding_rules
            __props__['healthcheck'] = healthcheck
            __props__['name'] = name
            __props__['redirect_http_to_https'] = redirect_http_to_https
            if region is None:
                raise TypeError("Missing required property 'region'")
            __props__['region'] = region
            __props__['sticky_sessions'] = sticky_sessions
            __props__['ip'] = None
            __props__['status'] = None
            __props__['urn'] = None
        super(LoadBalancer, __self__).__init__(
            'digitalocean:index/loadBalancer:LoadBalancer',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, algorithm=None, droplet_ids=None, droplet_tag=None, enable_proxy_protocol=None, forwarding_rules=None, healthcheck=None, ip=None, name=None, redirect_http_to_https=None, region=None, status=None, sticky_sessions=None, urn=None):
        """
        Get an existing LoadBalancer resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] algorithm: The load balancing algorithm used to determine
               which backend Droplet will be selected by a client. It must be either `round_robin`
               or `least_connections`. The default value is `round_robin`.
        :param pulumi.Input[list] droplet_ids: A list of the IDs of each droplet to be attached to the Load Balancer.
        :param pulumi.Input[str] droplet_tag: The name of a Droplet tag corresponding to Droplets to be assigned to the Load Balancer.
        :param pulumi.Input[bool] enable_proxy_protocol: A boolean value indicating whether PROXY
               Protocol should be used to pass information from connecting client requests to
               the backend service. Default value is `false`.
        :param pulumi.Input[list] forwarding_rules: A list of `forwarding_rule` to be assigned to the
               Load Balancer. The `forwarding_rule` block is documented below.
        :param pulumi.Input[dict] healthcheck: A `healthcheck` block to be assigned to the
               Load Balancer. The `healthcheck` block is documented below. Only 1 healthcheck is allowed.
        :param pulumi.Input[str] name: The Load Balancer name
        :param pulumi.Input[bool] redirect_http_to_https: A boolean value indicating whether
               HTTP requests to the Load Balancer on port 80 will be redirected to HTTPS on port 443.
               Default value is `false`.
        :param pulumi.Input[str] region: The region to start in
        :param pulumi.Input[dict] sticky_sessions: A `sticky_sessions` block to be assigned to the
               Load Balancer. The `sticky_sessions` block is documented below. Only 1 sticky_sessions block is allowed.
        :param pulumi.Input[str] urn: The uniform resource name for the Load Balancer

        The **forwarding_rules** object supports the following:

          * `certificate_id` (`pulumi.Input[str]`) - The ID of the TLS certificate to be used for SSL termination.
          * `entryPort` (`pulumi.Input[float]`) - An integer representing the port on which the Load Balancer instance will listen.
          * `entryProtocol` (`pulumi.Input[str]`) - The protocol used for traffic to the Load Balancer. The possible values are: `http`, `https`, `http2` or `tcp`.
          * `targetPort` (`pulumi.Input[float]`) - An integer representing the port on the backend Droplets to which the Load Balancer will send traffic.
          * `targetProtocol` (`pulumi.Input[str]`) - The protocol used for traffic from the Load Balancer to the backend Droplets. The possible values are: `http`, `https`, `http2` or `tcp`.
          * `tlsPassthrough` (`pulumi.Input[bool]`) - A boolean value indicating whether SSL encrypted traffic will be passed through to the backend Droplets. The default value is `false`.

        The **healthcheck** object supports the following:

          * `checkIntervalSeconds` (`pulumi.Input[float]`) - The number of seconds between between two consecutive health checks. If not specified, the default value is `10`.
          * `healthyThreshold` (`pulumi.Input[float]`) - The number of times a health check must pass for a backend Droplet to be marked "healthy" and be re-added to the pool. If not specified, the default value is `5`.
          * `path` (`pulumi.Input[str]`) - The path on the backend Droplets to which the Load Balancer instance will send a request.
          * `port` (`pulumi.Input[float]`) - An integer representing the port on the backend Droplets on which the health check will attempt a connection.
          * `protocol` (`pulumi.Input[str]`) - The protocol used for health checks sent to the backend Droplets. The possible values are `http` or `tcp`.
          * `responseTimeoutSeconds` (`pulumi.Input[float]`) - The number of seconds the Load Balancer instance will wait for a response until marking a health check as failed. If not specified, the default value is `5`.
          * `unhealthyThreshold` (`pulumi.Input[float]`) - The number of times a health check must fail for a backend Droplet to be marked "unhealthy" and be removed from the pool. If not specified, the default value is `3`.

        The **sticky_sessions** object supports the following:

          * `cookieName` (`pulumi.Input[str]`) - The name to be used for the cookie sent to the client. This attribute is required when using `cookies` for the sticky sessions type.
          * `cookieTtlSeconds` (`pulumi.Input[float]`) - The number of seconds until the cookie set by the Load Balancer expires. This attribute is required when using `cookies` for the sticky sessions type.
          * `type` (`pulumi.Input[str]`) - An attribute indicating how and if requests from a client will be persistently served by the same backend Droplet. The possible values are `cookies` or `none`. If not specified, the default value is `none`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["algorithm"] = algorithm
        __props__["droplet_ids"] = droplet_ids
        __props__["droplet_tag"] = droplet_tag
        __props__["enable_proxy_protocol"] = enable_proxy_protocol
        __props__["forwarding_rules"] = forwarding_rules
        __props__["healthcheck"] = healthcheck
        __props__["ip"] = ip
        __props__["name"] = name
        __props__["redirect_http_to_https"] = redirect_http_to_https
        __props__["region"] = region
        __props__["status"] = status
        __props__["sticky_sessions"] = sticky_sessions
        __props__["urn"] = urn
        return LoadBalancer(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class GetImageResult:
    """
    A collection of values returned by getImage.
    """
    def __init__(__self__, created=None, distribution=None, error_message=None, id=None, image=None, min_disk_size=None, name=None, private=None, regions=None, size_gigabytes=None, slug=None, source=None, status=None, tags=None, type=None):
        if created and not isinstance(created, str):
            raise TypeError("Expected argument 'created' to be a str")
        __self__.created = created
        if distribution and not isinstance(distribution, str):
            raise TypeError("Expected argument 'distribution' to be a str")
        __self__.distribution = distribution
        """
        The name of the distribution of the OS of the image.
        * `min_disk_size`: The minimum 'disk' required for the image.
        * `size_gigabytes`: The size of the image in GB.
        """
        if error_message and not isinstance(error_message, str):
            raise TypeError("Expected argument 'error_message' to be a str")
        __self__.error_message = error_message
        if id and not isinstance(id, float):
            raise TypeError("Expected argument 'id' to be a float")
        __self__.id = id
        if image and not isinstance(image, str):
            raise TypeError("Expected argument 'image' to be a str")
        __self__.image = image
        """
        The id of the image (legacy parameter).
        """
        if min_disk_size and not isinstance(min_disk_size, float):
            raise TypeError("Expected argument 'min_disk_size' to be a float")
        __self__.min_disk_size = min_disk_size
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        __self__.name = name
        if private and not isinstance(private, bool):
            raise TypeError("Expected argument 'private' to be a bool")
        __self__.private = private
        """
        Is image a public image or not. Public images represent
        Linux distributions or One-Click Applications, while non-public images represent
        snapshots and backups and are only available within your account.
        * `regions`: A set of the regions that the image is available in.
        * `tags`: A set of tags applied to the image
        * `created`: When the image was created
        * `status`: Current status of the image
        * `error_message`: Any applicable error message pertaining to the image
        """
        if regions and not isinstance(regions, list):
            raise TypeError("Expected argument 'regions' to be a list")
        __self__.regions = regions
        if size_gigabytes and not isinstance(size_gigabytes, float):
            raise TypeError("Expected argument 'size_gigabytes' to be a float")
        __self__.size_gigabytes = size_gigabytes
        if slug and not isinstance(slug, str):
            raise TypeError("Expected argument 'slug' to be a str")
        __self__.slug = slug
        if source and not isinstance(source, str):
            raise TypeError("Expected argument 'source' to be a str")
        __self__.source = source
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        __self__.status = status
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        __self__.tags = tags
        if type and not isinstance(type, str):
            raise TypeError("Expected argument 'type' to be a str")
        __self__.type = type
class AwaitableGetImageResult(GetImageResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetImageResult(
            created=self.created,
            distribution=self.distribution,
            error_message=self.error_message,
            id=self.id,
            image=self.image,
            min_disk_size=self.min_disk_size,
            name=self.name,
            private=self.private,
            regions=self.regions,
            size_gigabytes=self.size_gigabytes,
            slug=self.slug,
            source=self.source,
            status=self.status,
            tags=self.tags,
            type=self.type)

def get_image(id=None,name=None,slug=None,source=None,opts=None):
    """
    Use this data source to access information about an existing resource.

    :param float id: The id of the image
    :param str name: The name of the image.
    :param str slug: The slug of the official image.
    :param str source: Restrict the search to one of the following categories of images:
           - `all` - All images (whether public or private)
           - `applications` - One-click applications
           - `distributions` - Distributions
           - `user` - (Default) User (private) images
    """
    __args__ = dict()


    __args__['id'] = id
    __args__['name'] = name
    __args__['slug'] = slug
    __args__['source'] = source
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getImage:getImage', __args__, opts=opts).value

    return AwaitableGetImageResult(
        created=__ret__.get('created'),
        distribution=__ret__.get('distribution'),
        error_message=__ret__.get('errorMessage'),
        id=__ret__.get('id'),
        image=__ret__.get('image'),
        min_disk_size=__ret__.get('minDiskSize'),
        name=__ret__.get('name'),
        private=__ret__.get('private'),
        regions=__ret__.get('regions'),
        size_gigabytes=__ret__.get('sizeGigabytes'),
        slug=__ret__.get('slug'),
        source=__ret__.get('source'),
        status=__ret__.get('status'),
        tags=__ret__.get('tags'),
        type=__ret__.get('type'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class Cdn(pulumi.CustomResource):
    certificate_id: pulumi.Output[str]
    """
    ID of a DigitalOcean managed TLS certificate for use with custom domains
    """
    created_at: pulumi.Output[str]
    """
    The date and time when the CDN Endpoint was created.
    """
    custom_domain: pulumi.Output[str]
    """
    The fully qualified domain name (FQDN) of the custom subdomain used with the CDN Endpoint.
    """
    endpoint: pulumi.Output[str]
    """
    The fully qualified domain name (FQDN) from which the CDN-backed content is served.
    """
    origin: pulumi.Output[str]
    """
    The fully qualified domain name, (FQDN) for a Space.
    """
    ttl: pulumi.Output[float]
    """
    The time to live for the CDN Endpoint, in seconds. Default is 3600 seconds.
    * `certificate_id`- (Optional) The ID of a DigitalOcean managed TLS certificate used for SSL when a custom subdomain is provided.
    """
    def __init__(__self__, resource_name, opts=None, certificate_id=None, custom_domain=None, origin=None, ttl=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a DigitalOcean CDN Endpoint resource for use with Spaces.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-digitalocean/blob/master/website/docs/r/cdn.html.markdown.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] certificate_id: ID of a DigitalOcean managed TLS certificate for use with custom domains
        :param pulumi.Input[str] custom_domain: The fully qualified domain name (FQDN) of the custom subdomain used with the CDN Endpoint.
        :param pulumi.Input[str] origin: The fully qualified domain name, (FQDN) for a Space.
        :param pulumi.Input[float] ttl: The time to live for the CDN Endpoint, in seconds. Default is 3600 seconds.
               * `certificate_id`- (Optional) The ID of a DigitalOcean managed TLS certificate used for SSL when a custom subdomain is provided.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['certificate_id'] = certificate_id
            __props__['custom_domain'] = custom_domain
            if origin is None:
                raise TypeError("Missing required property 'origin'")
            __props__['origin'] = origin
            __props__['ttl'] = ttl
            __props__['created_at'] = None
            __props__['endpoint'] = None
        super(Cdn, __self__).__init__(
            'digitalocean:index/cdn:Cdn',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, certificate_id=None, created_at=None, custom_domain=None, endpoint=None, origin=None, ttl=None):
        """
        Get an existing Cdn resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] certificate_id: ID of a DigitalOcean managed TLS certificate for use with custom domains
        :param pulumi.Input[str] created_at: The date and time when the CDN Endpoint was created.
        :param pulumi.Input[str] custom_domain: The fully qualified domain name (FQDN) of the custom subdomain used with the CDN Endpoint.
        :param pulumi.Input[str] endpoint: The fully qualified domain name (FQDN) from which the CDN-backed content is served.
        :param pulumi.Input[str] origin: The fully qualified domain name, (FQDN) for a Space.
        :param pulumi.Input[float] ttl: The time to live for the CDN Endpoint, in seconds. Default is 3600 seconds.
               * `certificate_id`- (Optional) The ID of a DigitalOcean managed TLS certificate used for SSL when a custom subdomain is provided.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["certificate_id"] = certificate_id
        __props__["created_at"] = created_at
        __props__["custom_domain"] = custom_domain
        __props__["endpoint"] = endpoint
        __props__["origin"] = origin
        __props__["ttl"] = ttl
        return Cdn(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

